/*
 * Copyright (C) 2007 Jonathan Hudson <jh+gps@daria.co.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <assert.h>
#include <wait.h>
#include <unistd.h>
#include "wbt201.h"
#include "bluebus.h"
#include <gudev/gudev.h>

#define VENDOR_ID "10c4"
#define MODEL_ID  "ea60"
static G_rays* g;

static int get_ttyusb(char* devs)
{
    GUdevClient* uc;
    GUdevDevice* gu;
    static const char * const subsys[] = {"usb",NULL};
    int ndev = 0;
    uc = g_udev_client_new(subsys);
    if (uc)
    {
        char devname[32] = "/dev/ttyUSBx";
        int i;
        
        for(i=0; i < 10; i++)
        {
            devname[11]='0'+i;
            gu = g_udev_client_query_by_device_file(uc, devname);
            if(gu)
            {
                const char *val;
                val = g_udev_device_get_property(gu, "ID_VENDOR_ID");
                if( 0 == strcmp(val,VENDOR_ID))
                {
                    val = g_udev_device_get_property(gu, "ID_MODEL_ID");
                    if (0 == strcmp(val, MODEL_ID))
                    {
                        devs[ndev]=i;
                        ndev++;
                    }
                }
                g_object_unref(gu);
            }
        }
        g_object_unref(uc);
    }
    return ndev;
}

static void clean_bluez()
{
    g_ray_disconnect (g);
    g_ray_unref(g);
    g = NULL;
}


static void check_dev(char *dev)
{
    if(0 == strcasecmp(dev,"auto"))
    {
        char devs[16];
        memset(devs,0xff,sizeof(devs));
        if(get_ttyusb(devs) == 1)
        {
            wbt->curdev = g_malloc(32);
            sprintf(wbt->curdev,"/dev/ttyUSB%c", devs[0]+'0');
        }
    }

    if(wbt->curdev == NULL)
    {
        if( (strcasecmp(dev,"auto") == 0)  ||
            (strcasecmp(dev,"bluetooth") == 0) ||
            (strstr(dev,"rfcomm") != NULL))
        {
            GtkWidget *q;
            assert(NULL != (q=GTK_WIDGET (gtk_builder_get_object
                                          (wbt->builder,
                                          "button3"))));
            gtk_button_set_use_stock(GTK_BUTTON(q),TRUE);
            gtk_button_set_label (GTK_BUTTON(q),GTK_STOCK_CANCEL);
            if(g == NULL)
            {
                g = g_ray_new();
                if(g && g_ray_connect (g) == 0)
                {
                    char *gdev;
                    gtk_main();
                    gdev = g_ray_get_device (g);
                    if(gdev)
                    {
                        wbt->curdev = g_strdup(gdev);
                    }
                    else
                    {
                        puts("no con");
                        clean_bluez();
                    }
                }
            }
            gtk_button_set_use_stock(GTK_BUTTON(q),TRUE);
            gtk_button_set_label (GTK_BUTTON(q),GTK_STOCK_QUIT);
        }
        else
        {
            if(access(dev,R_OK|W_OK) == 0)        
                wbt->curdev = g_strdup(dev);
        }
    }
}

#if 0
void cleanup(void)
{
    if (wbt->serfd != -1)
    {
        int len;
        gdk_input_remove(wbt->gio);
        do {
            len = write(wbt->serfd, "@AL,2,1", sizeof("@AL,2,1")-1);
        } while (len == -1 && (errno == EAGAIN || errno == EINTR));
        tcdrain(wbt->serfd);
        close(wbt->serfd);
    }
}
#endif

void setup_serial(int fd,int baudrate)
{
    struct termios tio;   
    memset (&tio, 0, sizeof(tio));
    tio.c_cflag = CS8 | CLOCAL | CREAD | O_NDELAY | O_NONBLOCK;
    tio.c_iflag = IGNPAR;
    tio.c_oflag = 0;
    tio.c_lflag = 0;
    tio.c_cc[VTIME]    = 0;
    tio.c_cc[VMIN]     = 1;
    switch (baudrate)
    {
        case 4800:   baudrate=B4800; break;              
        case 9600:   baudrate=B9600; break;
        case 19200:  baudrate=B19200; break;
        case 38400:  baudrate=B38400; break;
        case 57600:  baudrate=B57600; break;
        case 115200: baudrate=B115200; break;
        case 230400: baudrate=B230400; break;
    }
    cfsetispeed(&tio,baudrate);
    cfsetospeed(&tio,baudrate);
    tcsetattr(fd,TCSANOW,&tio);
}

int init_serial (char *name, int baudrate)
{
    int fd =-1;
    
    if(wbt->curdev)
    {
        g_free(wbt->curdev);
        wbt->curdev = NULL;
    }
        
    check_dev(name);
    if(wbt->curdev)
    {
        fd = open(wbt->curdev, O_RDWR|O_NOCTTY|O_NONBLOCK);
        if (fd != -1)
        {
            setup_serial(fd,baudrate);
        }
        else
        {
            serial_tidy();
        }
    }
    set_menu_states(TRUE);
    return fd;
}

void serial_tidy(void)
{
    if(wbt->curdev)
    {
        if(g)
        {
            clean_bluez();
        }
        g_free(wbt->curdev);
        wbt->curdev = NULL;
    }
}

void close_serial(void)
{
    GtkWidget *q;

    set_menu_states(FALSE);
    assert(NULL != (q=GTK_WIDGET (gtk_builder_get_object (wbt->builder,"conn_button1"))));
    gtk_button_set_use_stock(GTK_BUTTON(q),TRUE);
    gtk_button_set_label (GTK_BUTTON(q),GTK_STOCK_CONNECT  );
    assert(NULL != (q = GTK_WIDGET (gtk_builder_get_object (wbt->builder,"conn_state1"))));
    gtk_image_set_from_stock(GTK_IMAGE(q), GTK_STOCK_NO, GTK_ICON_SIZE_MENU);
    gtk_main_iteration_do(FALSE);

    if ( wbt->serfd != -1)
    {
        g_io_channel_shutdown(wbt->gio, FALSE, NULL);
        g_io_channel_unref(wbt->gio);
        close(wbt->serfd);
        wbt->serfd = -1;
        serial_tidy();
    }
    wbt->connstat = 0;
}


gboolean read_data(GIOChannel *source, GIOCondition condition, gpointer data)
{
    static gchar statbuf[1024];
    gchar buf[129];
    int len;
    
    do {
        len = read(wbt->serfd, buf, sizeof(buf)-1);
    } while (len == -1 && (errno == EAGAIN || errno == EINTR));
    
    if(len > 0)
    {
        gchar *sp,*ptr;
        sp = buf;
        buf[len] = 0;
        while((ptr = index(sp, '\n')))
        {
            *ptr = 0;
            strcat(statbuf, sp);
            serial_process(statbuf);
            *statbuf = 0;
            sp = ptr+1;
        }
        if (*sp != '\0')
        {
            strcat(statbuf, sp);
        }
        return TRUE;
    }
    else 
    {
        close_serial();
        return FALSE;
    }
}
